@@ -1,5 +1,6 @@
 use crate::*;
 use byteorder::{LittleEndian, ReadBytesExt};
+use x25519_dalek::{EphemeralSecret, PublicKey};
 
 #[derive(Debug, PartialEq)]
 pub struct TeleportHeader {
@@ -100,29 +101,21 @@ impl TeleportHeader {
 #[derive(Copy, Clone, Debug, PartialEq)]
 pub struct TeleportEnc {
     secret: [u8; 32],
-    privkey: [u8; 64],
     remote: [u8; 32],
-    public: TeleportEcdhExchange,
-}
-
-#[derive(Copy, Clone, Debug, PartialEq)]
-pub struct TeleportEcdhExchange {
-    key: [u8; 32],
+    pub public: [u8; 32],
 }
 
 impl TeleportEnc {
     pub fn new() -> TeleportEnc {
-        let (privkey, pubkey) = crypto::genkey();
         TeleportEnc {
             secret: [0; 32],
             remote: [0; 32],
-            privkey,
-            public: TeleportEcdhExchange { key: pubkey },
+            public: [0; 32],
         }
     }
 
     pub fn serialize(self) -> Vec<u8> {
-        self.public.key.to_vec()
+        self.public.to_vec()
     }
 
     pub fn deserialize(&mut self, input: &[u8]) -> Result<(), Error> {
@@ -135,13 +128,12 @@ impl TeleportEnc {
 
         self.remote = input[..32].try_into().expect("Error reading public key");
 
-        self.calc_secret();
-
         Ok(())
     }
 
-    pub fn calc_secret(&mut self) {
-        self.secret = crypto::calc_secret(&self.remote, &self.privkey)
+    pub fn calc_secret(&mut self, privkey: EphemeralSecret) {
+        let pubkey = PublicKey::from(self.remote);
+        self.secret = privkey.diffie_hellman(&pubkey).to_bytes()
     }
 
     pub fn encrypt(self, nonce: &[u8; 12], input: &[u8]) -> Result<Vec<u8>, Error> {
@@ -377,28 +369,28 @@ impl TeleportInitAck {
 #[derive(Clone, Debug, PartialEq)]
 pub struct TeleportDelta {
     pub filesize: u64,
-    pub checksum: Hash,
+    pub hash: u64,
     pub chunk_size: u64,
-    delta_checksum_len: u16,
-    pub delta_checksum: Vec<Hash>,
+    chunk_hash_len: u16,
+    pub chunk_hash: Vec<u64>,
 }
 
 impl TeleportDelta {
     pub fn new() -> TeleportDelta {
         TeleportDelta {
             filesize: 0,
-            checksum: [0; 32].try_into().unwrap(),
+            hash: 0,
             chunk_size: 0,
-            delta_checksum_len: 0,
-            delta_checksum: Vec::<Hash>::new(),
+            chunk_hash_len: 0,
+            chunk_hash: Vec::<u64>::new(),
         }
     }
 
-    fn delta_serial(input: &[Hash]) -> Vec<u8> {
+    fn delta_serial(input: &[u64]) -> Vec<u8> {
         let mut out = Vec::<u8>::new();
 
         for i in input {
-            out.append(&mut i.as_bytes().to_vec());
+            out.append(&mut i.to_le_bytes().to_vec());
         }
 
         out
@@ -411,35 +403,36 @@ impl TeleportDelta {
         out.append(&mut self.filesize.to_le_bytes().to_vec());
 
         // Add file hash
-        out.append(&mut self.checksum.as_bytes().to_vec());
+        out.append(&mut self.hash.to_le_bytes().to_vec());
 
         // Add chunk size
         out.append(&mut self.chunk_size.to_le_bytes().to_vec());
 
         // Add delta vector length
-        let dlen = self.delta_checksum.len() as u16;
+        let dlen = self.chunk_hash.len() as u16;
         out.append(&mut dlen.to_le_bytes().to_vec());
 
         // Add delta vector
-        out.append(&mut TeleportDelta::delta_serial(&self.delta_checksum));
+        out.append(&mut TeleportDelta::delta_serial(&self.chunk_hash));
 
         out
     }
 
-    fn delta_deserial(input: &[u8]) -> Result<Vec<Hash>, Error> {
-        if input.len() % 32 != 0 {
+    fn delta_deserial(input: &[u8], len: u16) -> Result<Vec<u64>, Error> {
+        if input.len() % 8 != 0 || len as usize != input.len() / 8 {
             return Err(Error::new(
                 ErrorKind::InvalidData,
-                "Cannot deserialize Vec<Hash>",
+                "Cannot deserialize Vec<u64>",
             ));
         }
 
-        let mut out = Vec::<Hash>::new();
-
-        for i in input.chunks(32) {
-            let a: [u8; 32] = i.try_into().unwrap();
-            let h: Hash = a.try_into().unwrap();
-            out.push(h);
+        let mut out = Vec::<u64>::new();
+        let mut buf = input;
+        let mut count: u16 = len;
+        while count > 0 {
+            let a: u64 = buf.read_u64::<LittleEndian>().unwrap();
+            out.push(a);
+            count -= 1;
         }
 
         Ok(out)
@@ -448,7 +441,7 @@ impl TeleportDelta {
     pub fn deserialize(&mut self, input: &[u8]) -> Result<(), Error> {
         let mut buf: &[u8] = input;
 
-        if input.len() < 50 {
+        if input.len() < 26 {
             return Err(Error::new(
                 ErrorKind::InvalidData,
                 "Not enough data for Delta deserialize",
@@ -458,18 +451,16 @@ impl TeleportDelta {
         self.filesize = buf.read_u64::<LittleEndian>().unwrap();
 
         // Extract file hash
-        let csum: [u8; 32] = input[8..40].try_into().unwrap();
-        self.checksum = csum.try_into().unwrap();
-        let mut buf: &[u8] = &input[40..];
+        self.hash = buf.read_u64::<LittleEndian>().unwrap();
 
         // Extract chunk size
         self.chunk_size = buf.read_u64::<LittleEndian>().unwrap();
 
         // Extract delta vector length
-        self.delta_checksum_len = buf.read_u16::<LittleEndian>().unwrap();
+        self.chunk_hash_len = buf.read_u16::<LittleEndian>().unwrap();
 
         // Extract delta vector
-        self.delta_checksum = TeleportDelta::delta_deserial(&input[50..]).unwrap();
+        self.chunk_hash = TeleportDelta::delta_deserial(buf, self.chunk_hash_len).unwrap();
 
         Ok(())
     }
@@ -545,8 +536,7 @@ mod tests {
         101,
     ];
     const TESTDELTA: &[u8] = &[
-        177, 104, 222, 58, 0, 0, 0, 0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
-        5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 21, 205, 91, 7, 0, 0, 0, 0, 0, 0,
+        177, 104, 222, 58, 0, 0, 0, 0, 57, 48, 0, 0, 0, 0, 0, 0, 21, 205, 91, 7, 0, 0, 0, 0, 0, 0,
     ];
     const TESTDATAPKT: &[u8] = &[49, 212, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 1, 2, 3, 4, 5];
     const TESTINITACK: &[u8] = &[0, 0, 0, 6, 0, 0, 0, 5, 0, 0, 0];
@@ -577,11 +567,14 @@ mod tests {
         let mut a = TeleportEnc::new();
         let mut b = TeleportEnc::new();
 
+        let priva = crypto::genkey(&mut a);
+        let privb = crypto::genkey(&mut b);
+
         a.deserialize(&b.serialize()).unwrap();
         b.deserialize(&a.serialize()).unwrap();
 
-        a.calc_secret();
-        b.calc_secret();
+        a.calc_secret(priva);
+        b.calc_secret(privb);
 
         assert_eq!(a.secret, b.secret);
     }
@@ -594,11 +587,15 @@ mod tests {
         let mut a = TeleportEnc::new();
         let mut b = TeleportEnc::new();
 
+        let priva = crypto::genkey(&mut a);
+        let privb = crypto::genkey(&mut b);
+
         a.deserialize(&b.serialize()).unwrap();
         b.deserialize(&a.serialize()).unwrap();
 
-        a.calc_secret();
-        b.calc_secret();
+        a.calc_secret(priva);
+        b.calc_secret(privb);
+
         assert_eq!(a.secret, b.secret);
 
         let data = TESTHEADER.to_vec();
@@ -643,9 +640,9 @@ mod tests {
     fn test_teleportdelta_serialize() {
         let mut test = TeleportDelta::new();
         test.filesize = 987654321;
-        test.checksum = [5; 32].try_into().unwrap();
+        test.hash = 12345;
         test.chunk_size = 123456789;
-        test.delta_checksum = Vec::<Hash>::new();
+        test.chunk_hash = Vec::<u64>::new();
 
         let out = test.serialize();
 
@@ -656,9 +653,9 @@ mod tests {
     fn test_teleportdelta_deserialize() {
         let mut test = TeleportDelta::new();
         test.filesize = 987654321;
-        test.checksum = [5; 32].try_into().unwrap();
+        test.hash = 12345;
         test.chunk_size = 123456789;
-        test.delta_checksum = Vec::<Hash>::new();
+        test.chunk_hash = Vec::<u64>::new();
 
         let mut t = TeleportDelta::new();
         t.deserialize(TESTDELTA).unwrap();
