@@ -3,6 +3,8 @@ use crate::teleport::{TeleportAction, TeleportEnc, TeleportFeatures, TeleportHea
 use crate::*;
 use byteorder::{LittleEndian, ReadBytesExt};
 use rand::prelude::*;
+use std::hash::Hasher;
+use xxhash_rust::xxh3;
 
 struct SizeUnit {
     value: f64,
@@ -143,7 +145,7 @@ pub fn recv_packet(
 fn gen_chunk_size(file_size: u64) -> usize {
     let mut chunk = 1024;
     loop {
-        if file_size / chunk > 150 {
+        if file_size / chunk > 2048 {
             chunk *= 2;
         } else {
             break;
@@ -154,8 +156,8 @@ fn gen_chunk_size(file_size: u64) -> usize {
 }
 
 // Called from client
-pub fn calc_file_hash(filename: String) -> Result<Hash, Error> {
-    let mut hasher = blake3::Hasher::new();
+pub fn calc_file_hash(filename: String) -> Result<u64, Error> {
+    let mut hasher = xxh3::Xxh3::new();
     let mut buf = Vec::<u8>::new();
 
     let mut file = File::open(filename)?;
@@ -175,12 +177,12 @@ pub fn calc_file_hash(filename: String) -> Result<Hash, Error> {
             break;
         }
 
-        hasher.update(&buf);
+        hasher.write(&buf);
     }
 
     file.seek(SeekFrom::Start(0))?;
 
-    Ok(hasher.finalize())
+    Ok(hasher.finish())
 }
 
 pub fn add_feature(opt: &mut Option<u32>, add: TeleportFeatures) -> Result<(), Error> {
@@ -212,11 +214,11 @@ pub fn calc_delta_hash(mut file: &File) -> Result<teleport::TeleportDelta, Error
     file.seek(SeekFrom::Start(0))?;
     let mut buf = Vec::<u8>::new();
     buf.resize(gen_chunk_size(meta.len()), 0);
-    let mut hasher = blake3::Hasher::new();
-    let mut whole_hasher = blake3::Hasher::new();
-    let mut delta_checksum = Vec::<Hash>::new();
+    let mut whole_hasher = xxh3::Xxh3::new();
+    let mut chunk_hash = Vec::<u64>::new();
 
     loop {
+        let mut hasher = xxh3::Xxh3::new();
         // Read a chunk of the file
         let len = match file.read(&mut buf) {
             Ok(l) => l,
@@ -226,18 +228,17 @@ pub fn calc_delta_hash(mut file: &File) -> Result<teleport::TeleportDelta, Error
             break;
         }
 
-        hasher.update(&buf);
-        delta_checksum.push(hasher.finalize());
-        hasher.reset();
+        hasher.write(&buf);
+        chunk_hash.push(hasher.finish());
 
-        whole_hasher.update(&buf);
+        whole_hasher.write(&buf);
     }
 
     let mut out = teleport::TeleportDelta::new();
     out.filesize = file_size as u64;
     out.chunk_size = buf.len() as u64;
-    out.checksum = whole_hasher.finalize();
-    out.delta_checksum = delta_checksum;
+    out.hash = whole_hasher.finish();
+    out.chunk_hash = chunk_hash;
 
     file.seek(SeekFrom::Start(0))?;
 
